#ifndef FASTICA_H_UNIVERSITY_OF_OREGON_NIC
#define FASTICA_H_UNIVERSITY_OF_OREGON_NIC

class FastICAReader;

#include "LAPACKWrapper.h"

#include "MPIWrapper.h"
#include "RingBlock.h"

#define USE_FASTICA_VERBOSEx

#include <cmath>
#include <iostream>

#include "FastICAReader.h"
#include "Processor.h"
#include "nic_matrix.h"
#include "nic_vector.h"

#include "MersenneTwister.h"

#ifdef USE_TAU_PROFILING
#include <tau.h>
#endif

//------------------------------------------------------------------------------
extern "C" int dger_(int *, int *, double *, double *, int *, double *, int *,
                         double *, int *);

//------------------------------------------------------------------------------
extern void cubic(nic_matrix*, nic_vector*, nic_vector*);

extern void gaussian(nic_matrix*, nic_vector*, nic_vector*);

extern void hyp_tan(nic_matrix*, nic_vector*, nic_vector*);

//------------------------------------------------------------------------------

class FastICA : public Processor
{
public:
    // passed to constructor
    nic_matrix* data;

    // loaded by FastICAReader
    double tolerance;
    int    max_iterations;
    int    max_retries;
    int    steps;
    double error;
    bool   m_useRandomWeights;

    void   (*contrast)(nic_matrix*, nic_vector*, nic_vector*);
    nic_matrix* init_matrix;

    // created by constructor
    int n_channels;
    int n_observations;

    nic_vector* w;
    nic_vector* w_plus;
    nic_vector* w_temp;
    nic_matrix* BBt;

    MersenneTwister m_randNumGen;

private:
    nic_matrix* m_sph;

    RingBlock m_rb;
public:

    int rank;
    int total_size;

    FastICA(
        nic_matrix* d, FastICAReader * reader, 
        nic_matrix* sph, 
        unsigned long seed = 123456UL ); // the default seed value has no meaning

    ~FastICA();

    void run();

protected:

    void search_for_weights();
    void initialize_w (int i);
    void convergence(int i);

    double estimate_convergence( nic_vector* W, nic_vector* W_plus);

    void load_init_matrix(FastICAReader * reader, nic_matrix* sph);
    void load_contrast(char contrast_type);
    void validate_load();

};

#endif
// FASTICA_H_UNIVERSITY_OF_OREGON_NIC
