#ifndef FASTICANEW_H_UNIVERSITY_OF_OREGON_NIC
#define FASTICANEW_H_UNIVERSITY_OF_OREGON_NIC

#include "nic_matrix.h"
#include "nic_vector.h"
#include "FastICASettings.h"
#include "MersenneTwister.h"
#include "LAPACKWrapper.h"

extern "C" int dger_(int *, int *, double *, double *, int *, double *, int *,
                         double *, int *);

class FastICANew
{
public:
    FastICANew( 
        const FastICASettings& settings, 
        nic_matrix* data,
        unsigned long seed = 123456UL,
        bool verbose = false );
    ~FastICANew();

    void mixingMatrix( nic_matrix& matrix, bool spheringApplied = true );
    void setWhiteningMatrix( nic_matrix* matrix );

    void runFastICA();

private:
    // private methods
    void initialize( nic_vector& w, int channel, int retries );
    void loadGuess();
    double estimateConvergence( nic_vector& w, nic_vector& w1 );

    //private members
    // settings
    FastICASettings m_settings;
    
    // data
    nic_matrix* m_data;
    nic_matrix m_mixingMatrix;
    nic_matrix* m_whiteningMatrix;
    nic_matrix* m_invWhiteningMatrix;
    int m_channels; // rows of data
    int m_samples;  // columns of data
    nic_matrix* m_initialGuess;

    // function objects and functions
    MersenneTwister m_randNumGen;
    ContrastFunctions::ContrastFunction m_contrastFunction;

    // MPI information
    int m_commRank;
    int m_commSize;

    bool m_verbose;

};

#endif
// FASTICANEW_H_UNIVERSITY_OF_OREGON_NIC
