#ifndef FASTICAREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define FASTICAREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include "FastICAReader.h"

// DEFINE FORM DATA NAMES HERE 
#define N_FASTICA_PHRASES 8
char * FASTICA_PHRASES[N_FASTICA_PHRASES] = {
  "Convergence tolerance",
  "Maximum iterations",
  "Contrast function",
  "Initialization Type",
  "User defined source",
  "User defined type",
  "User defined orientation",
  "Retries"
};


FastICAReader::FastICAReader( std::istream& inStream )
:   Reader( inStream )
{
    phrase = FASTICA_PHRASES;
    n_phrases = N_FASTICA_PHRASES;
}

void FastICAReader::get_value()
{
    char * s;
    char c;
    int n;

    switch (current_phrase) 
    {
        case 0 : // Convergence tolerance
            tolerance = get_double(value);
            break;

        case 1 : // Maximum iterations
            iterations = get_int(value);
            break;

        case 2 : // Contrast function
            contrast = get_char(value);
            break;

        case 3 : // Initialization type
            initialization_type = get_char(value);
            break;

        case 4 : // User defined source
            s = get_str(value);
            n = strlen(s);
            if (n > 0) 
            {
                user_source = new char [n + 25];
                sprintf(user_source, "%s", s);
            } else 
            {
                user_source = NULL;
            }
            break;

        case 5 : // User defined type
            user_type = get_char(value);
            break;
        case 6 : // User defined orientation
            user_orient = get_char(value);
            break;
        case 7: // Number of retries
            m_numRetries = get_int( value );
            break;
        default :
            break;
    }
}

void FastICAReader::load_user_init(nic_matrix* init_mat)
{
    if (user_type == 'B' || user_type == 'b')
    {  // binary file
        if (user_orient == 'C' || user_orient == 'c')
        {
            BinaryMatrix * mat_reader =
                new BinaryMatrix (init_mat, user_source);
        }
        else if (user_orient == 'O' || user_orient == 'o')
        {
            BinaryMatrixT * mat_reader =
                new BinaryMatrixT (init_mat, user_source, false, 0 );
        }
    }
    else if ( user_type == 'A' || user_type == 'a' )
    {
        if (user_orient == 'C' || user_orient == 'c')
        {
            ASCIIMatrix * mat_reader = 
                new ASCIIMatrix (init_mat, user_source);
        }
        else if (user_orient == 'O' || user_orient == 'o')
        {
            ASCIIMatrixT * mat_reader = 
                new ASCIIMatrixT (init_mat, user_source);
        }
    }
    else
    {
        // error 
    }
}


#endif
// FASTICAREADER_CPP_UNIVERSITY_OF_OREGON_NIC
