#ifndef ICAOUTPUT_CPP_UNIVERSITY_OF_OREGON_NIC
#define ICAOUTPUT_CPP_UNIVERSITY_OF_OREGON_NIC

#include "ICAOutput.h"

ICAOutput::ICAOutput(
        Preprocessor * preprocessor, Processor * processor, 
	    ICAOutputReader * output_reader, CleanerReader::PROCESS_TYPE t)
{
    // initialize member pointers to null
    data = 0;
    weights = 0;
    unmixing = 0;
    q = 0;
    lambda = 0;
    avg = 0;
    cov = 0;
    sph = 0;

    m_binaryFileFormat = output_reader->m_binaryFileFormat;

    type = t;
    monitor_conv = output_reader->monitor_conv;
    processor->monitor_conv = monitor_conv;
    preprocessor->monitor_conv = monitor_conv;

    int outnamelen = strlen( output_reader->output_name );
    output_name = new char[ outnamelen + 1 ];
    strcpy( output_name, output_reader->output_name );
    //    output_name = output_reader->output_name;
    scr_wgh = output_reader->scr_wgh;
    scr_mix = output_reader->scr_mix;
    scr_umx = output_reader->scr_umx;
    scr_q = output_reader->scr_q;
    scr_lam = output_reader->scr_lam;
    scr_avg = output_reader->scr_avg;
    scr_cov = output_reader->scr_cov;
    scr_sph = output_reader->scr_sph;
    file_wgh = output_reader->file_wgh;
    file_mix = output_reader->file_mix;
    file_umx = output_reader->file_umx;
    file_q = output_reader->file_q;
    file_lam = output_reader->file_lam;
    file_avg = output_reader->file_avg;
    file_cov = output_reader->file_cov;
    file_sph = output_reader->file_sph;
    idpCompFileName = output_reader->ICFileName;
}

ICAOutput::~ICAOutput()
{
    delete [] output_name;
    if ( unmixing != NULL ) delete unmixing;
    if ( mixing != NULL ) delete mixing;
}

void ICAOutput::setup(Preprocessor * preprocessor, Processor * processor)
{
    avg = preprocessor->avg;
    cov = preprocessor->cov;
    sph = preprocessor->sph;
    q = preprocessor->q;
    lambda = preprocessor->lambda;
    weights = processor->weights;

    // here we have the whitened data (assuming 
    // after the preprocessor stage we have the data whitened)
    data = preprocessor->data; 
    unmixing = NULL;
    mixing = NULL;
    
    usingInfomax = preprocessor->m_useInfomaxSphering;
    no_cpus = preprocessor->n_cpus;

    n_channels = preprocessor->n_rows;

}

void ICAOutput::run( int rank )
{
    bool generateIndependentComponents = ( idpCompFileName.length() != 0 );
    bool computeMPV = generateIndependentComponents && usingInfomax;

    // array of mean projected variance of the size of no of channels
    // for the current node
    double*     mean_projected_variance = 0; 

    // mean projected variance of the components as a whole
    double*     sum_projected_variance = 0;  


    int         ch = data->rows;
    int         samples = data->columns;

    #ifdef USE_TAU_PROFILING
        #ifdef __DISTRIBUTED
        if ( rank == 0 )
        {
        #endif
            TAU_PROFILE("ICAOutput::run()", "void(int)", TAU_DEFAULT);
        #ifdef __DISTRIBUTED
        }
        #endif
    #endif

    // the independent component data is stored in this array
    double* 		    comp_data = 0;
    nic_matrix* nic_comp_data = new nic_matrix;
    if ( generateIndependentComponents )
    {
        comp_data = new double[(data->rows*data->columns)]; 
        nic_comp_data->rows = data->rows;
        nic_comp_data->columns = data->columns;
        nic_comp_data->data = comp_data;
    }

    double* sumSquarePositive = 0;
    double* sumSquareNegative = 0;
    double* noPositive = 0;
    int*    invVector = 0;
    int*    shuffle = 0;

    if ( computeMPV ) 
    {
        sumSquarePositive = new double[data->rows];
        sumSquareNegative = new double[data->rows];
        noPositive = new double[data->rows];
        
        for(int i=0; i<data->rows; i++)
        {
            sumSquarePositive[i] = 0.0;
            sumSquareNegative[i] = 0.0;
            noPositive[i] = 0.0;
        }
        
        if ( rank==0 )
        {
            invVector = new int[data->rows];
            shuffle = new int[ch];
        }

        mean_projected_variance = new double[data->rows]; 
        sum_projected_variance = new double[data->rows];  
        for(int i=0;i<ch;i++) 
        {
            mean_projected_variance[i] = 0.0;
        }
    }

    char * base_name = 0;
    char * file_name = 0;
    // compute mixing matrix as sph^{-1}
    if( rank == 0) 
    {
        base_name = new char[strlen(output_name) + 15];
        file_name = new char[strlen(output_name) + 15 + 50];
           
        sprintf( base_name, "%s", output_name );
        if(monitor_conv) cout << "###output_prefix=" << base_name << endl;
    }
    // ensure that the user has write permission to the results directory.
    //    write_validate(base_name);

    /*CALCULATING MIXING AND UNMIXING MATRICES*/
    if(usingInfomax) 
    {
        compute_mixing();
        compute_unmixing();
    }

    /*CALCULATE THE INDEPENDENT COMPONENTS SHARE FOR THE CURRENT NODE*/
    // We have the whitened data from the output of the preprocessor
    //WE SHOULD ADD A OPTION SO AS TO COMPUTE THE INDEPENDENT COMPONENTS HERE
    if ( generateIndependentComponents ) 
    {
        for(int j=0;j<samples;j++) 
        {
            for(int i=0;i<ch;i++) 
            {
                comp_data[j*ch+i] = 0.0;
            }

            for(int i=0;i<ch;i++) 
            {
                for(int k=0;k<ch;k++) 
                {
                    comp_data[j*ch+k] += 
                        (weights->data)[i*ch+k]*(data->data)[j*ch+i];
                }
            }
        }
    }

    /*CALCULATE THE MEAN PROJECTED VARIANCE VECTOR AS A CONTRIBUTION OF THIS NODE*/
    double temp;
    if ( computeMPV ) 
    {   
        for(int j=0;j<samples;j++) 
        {
            for(int i=0;i<ch;i++) 
            {
                temp=0.0;
                for(int k=0;k<ch;k++)
                {
                    temp += pow((mixing->data)[i*ch+k],2);
                }
                if ( comp_data != 0 )
                {
                    temp *= pow(comp_data[j*ch+i],2);
                }
                
                mean_projected_variance[i] += (temp/(ch-1));
            }
        }
        
        for(int j=0;j<samples;j++)
        {
            for(int i=0;i<ch;i++)
            {
                if ( comp_data != 0 )
                {
                    if(comp_data[j*ch+i] >= 0.0)
                    {
                        sumSquarePositive[i] += pow(comp_data[j*ch+i],2);
                        noPositive[i] += 1;
                    }
                    else
                    {
                        sumSquareNegative[i] += pow(comp_data[j*ch+i],2);
                    }
                }
            }
        }

        #ifdef __DISTRIBUTED
        MPI_Allreduce(sumSquarePositive,sum_projected_variance, 
            ch, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
        MPI_Allreduce(sumSquareNegative,sum_projected_variance, 
            ch, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
        MPI_Allreduce(noPositive,sum_projected_variance, 
            ch, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
        #endif

        if(rank==0) 
        {            
            for(int i=0;i<ch;i++)
            {
                sumSquarePositive[i] /= noPositive[i];
                sumSquarePositive[i] = sqrt(sumSquarePositive[i]);
                sumSquareNegative[i] /= (samples-noPositive[i]);
                sumSquareNegative[i] = sqrt(sumSquareNegative[i]);
                if(sumSquarePositive[i]<sumSquareNegative[i])
                {
                    invVector[i] = -1;
                }
                else
                {
                    invVector[i] = 1;
                }
                
            }
            cout<<"\n";
        }

        for(int i = 0 ; i < ch; i++) 
        {
            mean_projected_variance[i] = (mean_projected_variance[i])/samples;
        }

        /*CALCULATE THE MEAN PROJECTED VARIANCE OF 
            THE WHOLE DATA USING MPI_All_Reduce*/
        #ifdef __DISTRIBUTED
        MPI_Allreduce(mean_projected_variance,sum_projected_variance, 
            ch, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
        for(int i=0;i<ch;i++) 
        {
            mean_projected_variance[i] = sum_projected_variance[i];
        }
        #endif

    }

    if ((scr_wgh == 'y' || scr_wgh == 'Y') && weights != NULL) 
    {
        cout << "Weight Matrix : " << endl;
        cout << *weights << endl;
    }

    if ((scr_mix == 'y' || scr_mix == 'Y') && mixing != NULL) 
    {
        if (mixing == NULL) 
        {
            if (compute_mixing()) 
            {
                cout << "Mixing Matrix : " << endl;
                cout << *mixing << endl;
            }
        }
        else 
        {
            cout << "Mixing Matrix : " << endl;
            cout << *mixing << endl;
        }
    }

    if ((scr_umx == 'y' || scr_umx == 'Y') && unmixing != NULL) 
    {
        if (unmixing == NULL) 
        {
            if (compute_unmixing()) 
            {
                cout << "Unmixing Matrix : " << endl;
                cout << *unmixing << endl;
            }
        }
        else 
        {
            cout << "Unmixing Matrix : " << endl;
            cout << *unmixing << endl;
        }

    }

    if ((scr_q == 'y' || scr_q == 'Y') && q != NULL) 
    {
        cout << "Q Matrix : " << endl;
        cout << *q << endl;
    }

    if ((scr_lam == 'y' || scr_lam == 'Y') && lambda != NULL) 
    {
        cout << "Lambda Vector : " << endl;
        cout << *lambda << endl;
    }

    if ((scr_avg == 'y' || scr_avg == 'Y') && avg != NULL) 
    {
        cout << "Average Vector : " << endl;
        cout << *avg << endl;
    }

    if ((scr_cov == 'y' || scr_cov == 'Y') && cov != NULL)
    {
        cout << "Covariance Matrix : " << endl;
        cout << *cov << endl;
    }

    if ((scr_sph == 'y' || scr_sph == 'Y') && sph != NULL) 
    {
        cout << "Sphering Matrix : " << endl;
        cout << *sph << endl;
    }

    // Now we have the array of mean projected 
    // variance according to which we can sort the weight matrix
    // CREATE SHUFFLE ARRAY ACCORDING TO MPV FROM LARGE TO SMALL
    if(rank==0) 
    {
        char space = ' ';
        if ( computeMPV )
        {
            for(int i=0;i<ch;i++) 
            {
                shuffle[i] = i;
            }

            //quicksort
            int s_temp;
            for(int i=0;i<ch;i++) 
            {
                for(int j=i;j<ch;j++) 
                {
                    if(mean_projected_variance[i]<mean_projected_variance[j]) 
                    {
                        temp = mean_projected_variance[i];
                        mean_projected_variance[i] = mean_projected_variance[j];
                        mean_projected_variance[j] = temp;
                
                        s_temp = shuffle[i];
                        shuffle[i] = shuffle[j];
                        shuffle[j] = s_temp; 
                    }
                }
            }

            if(monitor_conv) 
            {
                cout<<"SHUFFLE ARRAY : ";
                for(int i=0;i<ch;i++) 
                {
                    cout<<shuffle[i]<<" ";
                }
                cout<<'\n';
            }
        }
        //shuffle array created
        //SHUFFLE ARRAY IS GETTING COMPUTED CORRECTLY --- RESULTS CHECKED
            
        if ((file_wgh == 'a' || file_wgh == 'A' || 
            file_wgh == 'e' || file_wgh == 'E') && 
            weights != NULL) 
        {
        
            sprintf(file_name, "%s_ascii.wgt", base_name);
            if(usingInfomax)
            {
                ASCIIMatrixWriter* mat_reader = 
                    new ASCIIMatrixWriter(weights, file_name,space, 
                        shuffle, invVector, 0);
                //row shuffle in weights matrix
                delete mat_reader; // why not just a local variable?
            }
            else 
            {
                ASCIIMatrixWriter* mat_reader = 
                    new ASCIIMatrixWriter(weights, file_name);
                //row shuffle in weights matrix
                delete mat_reader; // why not just a local variable?
            }
        }

        if ((file_wgh == 'b' || file_wgh == 'B' || 
            file_wgh == 'e' || file_wgh == 'E') &&
            weights != NULL)
        {
            // switch to name mangler
            sprintf(file_name, "%s_binary.wgt", base_name);
            if(usingInfomax)
            {
                BinaryMatrixWriter* mat_reader =
                    new BinaryMatrixWriter(
                        (nic_matrix*)weights, file_name, 
                        shuffle, invVector, 0, m_binaryFileFormat );  
                    //Binary Matrix T Writer
                delete mat_reader; // why not ust use a local variable?
            }
            else
            {
                BinaryMatrixWriter* mat_reader =
                    new BinaryMatrixWriter(
                        (nic_matrix*)weights, file_name, 
                        0, 0, 0, m_binaryFileFormat );  
                    //Binary Matrix T Writer
                delete mat_reader; // why not ust use a local variable?
            }
        }

        if (file_mix == 'a' || file_mix == 'A' || 
            file_mix == 'e' || file_mix == 'E') 
        {
            if (mixing == NULL) 
            {
                if (compute_mixing()) 
                {
                    sprintf(file_name, "%s_ascii.mix", base_name);
                    if(usingInfomax)
                    {
                        ASCIIMatrixWriter* mat_reader = 
                            new ASCIIMatrixWriter(
                                mixing, file_name, space, 
                                shuffle, invVector, 1);
                            //column shuffle in mixing matrix
                        delete mat_reader;
                    }
                    else 
                    {
                        ASCIIMatrixWriter* mat_reader = 
                            new ASCIIMatrixWriter(mixing, file_name);
                            //column shuffle in mixing matrix
                        delete mat_reader;
                    }
                }
            }
            else 
            {
                sprintf(file_name, "%s_ascii.mix", base_name);
                if(usingInfomax)
                {
                    ASCIIMatrixWriter* mat_reader = 
                        new ASCIIMatrixWriter(
                            mixing, file_name, space, shuffle, invVector, 1);
                        //column shuffle in mixing matrix
                    delete mat_reader;
                }
                else 
                {
                    ASCIIMatrixWriter* mat_reader = 
                        new ASCIIMatrixWriter(mixing, file_name);
                        //column shuffle in mixing matrix
                    delete mat_reader;
                }
            }
        } 

        if (file_mix == 'b' || file_mix == 'B' || 
            file_mix == 'e' || file_mix == 'E') 
        {
            if (mixing == NULL) 
            {
                if (compute_mixing()) 
                {
                    sprintf(file_name, "%s_binary.mix", base_name);
                    if(usingInfomax)
                    {
                        BinaryMatrixWriter* mat_reader =
                            new BinaryMatrixWriter(
                                mixing, file_name, shuffle, invVector, 
                                1, m_binaryFileFormat);  
                            //Binary Matrix T Writer
                        delete mat_reader;
                    }
                    else
                    {
                        BinaryMatrixWriter* mat_reader =
                            new BinaryMatrixWriter(
                                mixing, file_name, 0, 0, 0, 
                                m_binaryFileFormat );  
                            //Binary Matrix T Writer
                        delete mat_reader;
                    }
                }
            }
            else
            {
                sprintf(file_name, "%s_binary.mix", base_name);
                if(usingInfomax)
                {
                    BinaryMatrixWriter* mat_reader =
                        new BinaryMatrixWriter(
                            mixing, file_name, shuffle, invVector, 
                            1, m_binaryFileFormat);  
                        //Binary Matrix T Writer
                    delete mat_reader;
                }
                else
                {
                    BinaryMatrixWriter* mat_reader =
                        new BinaryMatrixWriter(
                            mixing, file_name, 0, 0, 0, m_binaryFileFormat ); 
                            //Binary Matrix T Writer
                    delete mat_reader;
                }
            }
        } 

        if ((file_umx == 'a' || file_umx == 'A' || 
            file_umx == 'e' || file_umx == 'E')) 
        {
            if(unmixing == NULL) 
            {
                if(compute_unmixing()) 
                {
                    sprintf(file_name, "%s_ascii.umx", base_name);
                    if(usingInfomax)
                    {
                        ASCIIMatrixWriter* mat_reader = 
                            new ASCIIMatrixWriter(
                                unmixing, file_name, space, 
                                shuffle, invVector,  0);
                            //row shuffle in unmixing matrix
                        delete mat_reader;
                    }
                    else
                    {
                        ASCIIMatrixWriter* mat_reader = 
                            new ASCIIMatrixWriter(
                                unmixing, file_name);
                            //row shuffle in unmixing matrix       
                        delete mat_reader;
                    }
                }
            }
            else 
            {
                sprintf(file_name, "%s_ascii.umx", base_name);
                if(usingInfomax)
                {
                    ASCIIMatrixWriter* mat_reader = 
                        new ASCIIMatrixWriter(
                            unmixing, file_name, space, shuffle, 
                            invVector, 0);
                        //row shuffle in unmixing matrix
                    delete mat_reader;
                }
                else
                {
                    ASCIIMatrixWriter* mat_reader = 
                        new ASCIIMatrixWriter(unmixing, file_name);
                        //row shuffle in unmixing matrix 
                    delete mat_reader;
                }
            }
        } 

        if (file_umx == 'b' || file_umx == 'B' || 
            file_umx == 'e' || file_umx == 'E') 
        {
            if(unmixing == NULL) 
            {
                if(compute_unmixing()) 
                {
                    sprintf(file_name, "%s_binary.umx", base_name);
                    if(usingInfomax)
                    {
                        BinaryMatrixWriter* mat_reader =
                            new BinaryMatrixWriter(
                            unmixing, file_name, shuffle, 
                            invVector, 0, m_binaryFileFormat);  
                            //Binary Matrix T Writer
                        delete mat_reader;
                    }
                    else
                    {
                        BinaryMatrixWriter* mat_reader =
                            new BinaryMatrixWriter(
                                unmixing, file_name, 0, 0, 0, 
                                m_binaryFileFormat );  
                            //Binary Matrix T Writer
                        delete mat_reader;
                    }
                }
            }
            else 
            {
                sprintf(file_name, "%s_binary.umx", base_name);
                if(usingInfomax)
                {
                    BinaryMatrixWriter* mat_reader =
                        new BinaryMatrixWriter(
                            unmixing, file_name, shuffle, 
                            invVector, 0, m_binaryFileFormat);
                        //Binary Matrix T Writer
                    delete mat_reader;
                }
                else
                {
                    BinaryMatrixWriter* mat_reader =
                        new BinaryMatrixWriter(
                            unmixing, file_name, 
                            0, 0, 0, m_binaryFileFormat );
                        //Binary Matrix T Writer
                    delete mat_reader;
                }
            }
        } 

        if ((file_q == 'a' || file_q == 'A' || 
            file_q == 'e' || file_q == 'E') &&
            q != NULL) 
        {
            sprintf(file_name, "%s_ascii.q", base_name);
            ASCIIMatrixWriter* mat_reader = 
                new ASCIIMatrixWriter(q, file_name);
            delete mat_reader;
        } 

        if ((file_q == 'b' || file_q == 'B' || 
            file_q == 'e' || file_q == 'E') &&
            q != NULL) 
        {
            sprintf(file_name, "%s_binary.q", base_name);
            BinaryMatrixWriter* mat_reader =
                new BinaryMatrixWriter(
                    q, file_name, 0, 0, 0, m_binaryFileFormat );  
            delete mat_reader;
        } 

        if ((file_lam == 'a' || file_lam == 'A' || 
            file_lam == 'e' || file_lam == 'E') && 
            lambda != NULL) 
        {
            sprintf(file_name, "%s_ascii.lam", base_name);
            ASCIIVectorWriter* mat_reader = 
                new ASCIIVectorWriter(lambda, file_name);
            delete mat_reader;
        } 

        if ((file_lam == 'b' || file_lam == 'B' || 
            file_lam == 'e' || file_lam == 'E') &&
            lambda != NULL) 
        {
            sprintf(file_name, "%s_binary.lam", base_name);
            BinaryVectorWriter* mat_reader =
                new BinaryVectorWriter(
                    lambda, file_name, m_binaryFileFormat );  
            delete mat_reader;
        } 

        if ((file_avg == 'a' || file_avg == 'A' || 
            file_avg == 'e' || file_avg == 'E') && 
            avg != NULL) 
        {
            sprintf(file_name, "%s_ascii.avg", base_name);
            ASCIIVectorWriter* mat_reader = 
                new ASCIIVectorWriter(avg, file_name);
            delete mat_reader;
        } 
    
        if ((file_avg == 'b' || file_avg == 'B' || 
            file_avg == 'e' || file_avg == 'E') &&
            avg != NULL) 
        {
            sprintf(file_name, "%s_binary.avg", base_name);
            BinaryVectorWriter* mat_reader =
                new BinaryVectorWriter (
                    avg, file_name, m_binaryFileFormat );  
            delete mat_reader;
        } 

        if ((file_cov == 'a' || file_cov == 'A' || 
            file_cov == 'e' || file_cov == 'E') &&
            cov != NULL) 
        {
            sprintf(file_name, "%s_ascii.cov", base_name);
            ASCIIMatrixWriter* mat_reader = 
                new ASCIIMatrixWriter(cov, file_name);
            delete mat_reader;
        } 

        if ((file_cov == 'b' || file_cov == 'B' || 
            file_cov == 'e' || file_cov == 'E') &&
            cov != NULL) 
        {
            sprintf(file_name, "%s.cov_binary", base_name);
            BinaryMatrixWriter* mat_reader =
                new BinaryMatrixWriter(
                    cov, file_name, 0, 0, 0, m_binaryFileFormat );  
            delete mat_reader;
        } 

        if ((file_sph == 'a' || file_sph == 'A' || 
            file_sph == 'e' || file_sph == 'E') &&
            sph != NULL) 
        {
            sprintf(file_name, "%s_ascii.sph", base_name);
            ASCIIMatrixWriter* mat_reader = 
                new ASCIIMatrixWriter(sph, file_name);
            delete mat_reader;
        } 

        if ((file_sph == 'b' || file_sph == 'B' || 
            file_sph == 'e' || file_sph == 'E') && 
            sph != NULL) 
        {
            sprintf(file_name, "%s_binary.sph", base_name);
            BinaryMatrixWriter* mat_reader =
                new BinaryMatrixWriter(
                    sph, file_name, 0, 0, 0, m_binaryFileFormat );  
            delete mat_reader;
        } 
        
        if((generateIndependentComponents) && usingInfomax)
        {   
            PartitionedDataset ICstoredata;
            ICstoredata.storeFile(
                rank, nic_comp_data, idpCompFileName, 1, shuffle, invVector);
        }
        if((generateIndependentComponents) && !usingInfomax)
        {
            PartitionedDataset ICstoredata;
            ICstoredata.storeFile(
                rank, nic_comp_data, idpCompFileName, no_cpus);
        }
    }
    if(rank!=0) 
    {
        if((generateIndependentComponents) && usingInfomax)
        {
            PartitionedDataset ICstoredata;
            ICstoredata.storeFile(rank, nic_comp_data, idpCompFileName);
        }
        if((generateIndependentComponents) && !usingInfomax)
        {
            PartitionedDataset ICstoredata;
            ICstoredata.storeFile(
                rank, nic_comp_data, idpCompFileName, no_cpus);
        }
    }

    // clean up the mess of allocated memory
    // delete[] comp_data; nic_comp_data owns this, so don't delete it
    delete nic_comp_data;
    delete[] sumSquarePositive;
    delete[] sumSquareNegative;
    delete[] noPositive;
    delete[] invVector;
    delete[] shuffle;
    delete[] mean_projected_variance;
    delete[] sum_projected_variance;
    delete[] base_name;
    delete[] file_name;

} // end of run()

bool ICAOutput::compute_mixing()
{
    bool rtn = false;
    CLAPACK_integer info;
    CLAPACK_integer lwork;
    double		worksize;
    CLAPACK_integer* ipiv = new CLAPACK_integer[ n_channels ];
    for ( int i = 0; i < n_channels; ++i ) ipiv[i] = 0;

    /**************************************************************
     If the unmixing matrix exists, copy it into the mixing matrix
     compute LU factorization
     if LU completes successfully, compute inverse
     if inverse completes successfull, return true;
     *************************************************************/
    if(mixing) 
    {
        rtn = true;
    } else if (compute_unmixing()) 
    {
        // copy the unmixing matrix into the mixing matrix
        mixing = new nic_matrix(*unmixing); 

        CLAPACK_integer	nChannelsLong = n_channels;
        
        // compute the LU factors    dgetrf.c
        dgetrf_(&nChannelsLong, &nChannelsLong, mixing->data, &nChannelsLong,
                ipiv, &info);

        // compute the inverse       dgetri.c
        if (info == 0) 
        {

	        // get workspace size
	        lwork = -1;
	        dgetri_(&nChannelsLong, mixing->data, &nChannelsLong, ipiv,
	            &worksize, &lwork, &info);

	        if (info == 0) 
            {
	            lwork = (CLAPACK_integer)(worksize); 
                // warning! from double to long
	            nic_vector work((int)worksize);
	            dgetri_(&nChannelsLong, mixing->data, &nChannelsLong, ipiv,
	                work.data, &lwork, &info);
	            rtn = true; 
	        } else 
            {
	            cout << "The matrix inversion algorithm failed. The mixing " 
                    << endl;
	            cout << "matrix was not computed." << endl;
	        }
        } else 
        {
            cout << "The LU decomposition failed for the mixing" << endl;
	        cout << "matrix. The mixing matrix was not computed." << endl;
        }
    } 
    else 
    {
        cout << "Could not produce mixing matrix because the" << endl;
        cout << "unmixing matrix could not be produced." << endl;
    }
    return rtn;
}

bool ICAOutput::compute_unmixing()
{ 
    bool rtn = false;

    /**************************************************************
     If the unmixing matrix exists, then it has already been computed
     there is no sense in recomputing it.
     If the unmixing matrix does not exist, but the weight and 
     sphering matricies do exist, then instantiate and compute the 
     unmixing matrix
     If the unmixing, weight and sphering matricies do not exist,
     then the unmixing matrix cannot be computed
     *************************************************************/
    if (unmixing) 
    {  // the mixing matrix has already been computed
        rtn = true;
    } 
    else if (weights != NULL && sph != NULL) 
    {
        if(monitor_conv) 
        {
	        cout << "COMPUTING UNMIXING MATRIX" << endl;
        }
        unmixing = new nic_matrix(n_channels, n_channels);
        // set unmixing matrix
        double alpha = 1;
        double beta = 0;

        char*       n = "N";
        char*       t = "T";

        if (type == CleanerReader::INFOMAX) 
        {
            dgemm_( n, n, &n_channels, &n_channels, &n_channels, &alpha, 
	            weights->data, &n_channels, sph->data, &n_channels, &beta, 
	            unmixing->data, &n_channels);
        } else if (type == CleanerReader::FASTICA)
        {
            dgemm_( t, n, &n_channels, &n_channels, &n_channels, &alpha, 
	            weights->data, &n_channels, sph->data, &n_channels, &beta, 
	            unmixing->data, &n_channels);
        }
        rtn = true;
    } else 
    {
        cout << "Unmixing matrix not produced. The weight matrix or the" 
            << endl;
        cout << "sphering matrix was missing" << endl;
    }
    return rtn;
}

void ICAOutput::validate_load()
{
    cout << "ICAOutput LOAD VALIDATION --------------------------------" << endl;
    cout << "\tweights    @ " << weights       << endl;
    cout << "\tunmixing   @ " << unmixing      << endl;
    cout << "\tmixing     @ " << mixing        << endl;
    cout << "\tq          @ " << q             << endl;
    cout << "\tlambda     @ " << lambda        << endl;
    cout << "\tavg        @ " << avg           << endl;
    cout << "\tcov        @ " << cov           << endl;
    cout << "\tsph        @ " << sph           << endl;

    cout << "output_name   = " << output_name  << endl;
    cout << "monitor_conv  = " << monitor_conv << endl;
    cout << "scr_wgh       = " << scr_wgh      << endl;
    cout << "scr_mix       = " << scr_mix      << endl;
    cout << "scr_umx       = " << scr_umx      << endl;
    cout << "scr_q         = " << scr_q        << endl;
    cout << "scr_lam       = " << scr_lam      << endl;
    cout << "scr_avg       = " << scr_avg      << endl;
    cout << "scr_cov       = " << scr_cov      << endl;
    cout << "scr_sph       = " << scr_sph      << endl;
    cout << "file_wgh      = " << file_wgh     << endl;
    cout << "file_mix      = " << file_mix     << endl;
    cout << "file_umx      = " << file_umx     << endl;
    cout << "file_q        = " << file_q       << endl;
    cout << "file_lam      = " << file_lam     << endl;
    cout << "file_avg      = " << file_avg     << endl;
    cout << "file_cov      = " << file_cov     << endl;
    cout << "file_sph      = " << file_sph     << endl;

    cout << "END ICAOutput LOAD VALIDATION ----------------------------" << endl;
}


#endif
// ICAOUTPUT_CPP_UNIVERSITY_OF_OREGON_NIC
