#ifndef ICAOUTPUT_H_UNIVERSITY_OF_OREGON_NIC
#define ICAOUTPUT_H_UNIVERSITY_OF_OREGON_NIC
/*******************************************************************************
NIC Signal Cleaning Toolbox                                                 ****
Author:  Kevin Glass                                                        ****
Copyright 2004                                                              ****
Department of Computer and Information Science, University of Oregon        ****
 ******************************************************************************/


#include "LAPACKWrapper.h"
#include "MPIWrapper.h"

#include <iostream>

#include "nic_vector.h"
#include "nic_matrix.h"
#include "Preprocessor.h"
#include "Processor.h"
#include "Output.h"
#include "ICAOutputReader.h"
#include "CleanerReader.h"
#include <math.h>

#include "ascii_vec_writer.h"
#include "ascii_matT_writer.h"
#include "ascii_mat_writer.h"
#include "binary_vec_writer.h"
#include "binary_mat_writer.h"

#ifdef USE_TAU_PROFILING
#include "TAU.h"
#endif

bool write_validate(char * file_name);

extern "C" int dgemm_(char *, char *, int *, int *, int *, 
		  double *, double *, int *, double *, 
		  int *, double *, double *, int *);

class ICAOutput : public Output 
{
public:
    nic_matrix* data;
    nic_matrix* weights;
    nic_matrix* unmixing;
    nic_matrix* mixing;
    nic_matrix* q;
    nic_vector* lambda;
    nic_vector* avg;
    nic_matrix* cov;
    nic_matrix* sph;
    int                  n_channels;

    char * output_name;
    bool monitor_conv;
    char scr_wgh;
    char scr_mix;
    char scr_umx;
    char scr_q;
    char scr_lam;
    char scr_avg;
    char scr_cov;
    char scr_sph;
    char file_wgh;
    char file_mix;
    char file_umx;
    char file_q;
    char file_lam;
    char file_avg;
    char file_cov;
    char file_sph;

    bool usingInfomax;
    std::string    idpCompFileName;
    int no_cpus;
    BinaryMatrixWriter::FileType        m_binaryFileFormat;

    CleanerReader::PROCESS_TYPE type;

    ICAOutput(
        Preprocessor * preprocessor, Processor * processor, 
	    ICAOutputReader * output_reader, CleanerReader::PROCESS_TYPE t);
    
    ~ICAOutput();
  
    void setup(Preprocessor * preprocessor, Processor * processor);

    void run(int rank);

  bool compute_mixing();
  bool compute_unmixing();
  void validate_load();
};

#endif
// ICAOUTPUT_H_UNIVERSITY_OF_OREGON_NIC
