#ifndef ICAOUTPUTREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define ICAOUTPUTREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include "ICAOutputReader.h"

// DEFINE FORM DATA NAMES HERE
#define N_ICA_OUTPUT_PHRASES 21
char * ICA_OUTPUT_PHRASES[N_ICA_OUTPUT_PHRASES] = 
{
  "Show convergence",
  "Weight matrix screen",
  "Mixing matrix screen",
  "Unmixing matrix screen",
  "Q matrix screen",
  "Lambda ^(-1/2) matrix screen",
  "Average vector screen",
  "Covariance matrix screen",
  "Sphering matrix screen",
  "File name",
  "Weight matrix file",
  "Mixing matrix file",
  "Unmixing matrix file",
  "Q matrix file",
  "Lambda ^(-1/2) matrix file",
  "Average vector file",
  "Covariance matrix file",
  "Sphering matrix file",
  "Validate form load",
  "Independent component file name",
  "Binary File Format"
};


ICAOutputReader::ICAOutputReader( std::istream& inStream )
:   Reader( inStream ),
    scr_sph( 'n' )
{
    m_binaryFileFormat = 'P';
    
    phrase = ICA_OUTPUT_PHRASES;
    n_phrases = N_ICA_OUTPUT_PHRASES;
    validate = false;
    output_name = 0;
}

ICAOutputReader::~ICAOutputReader()
{
    if ( output_name ) delete [] output_name;
}

void ICAOutputReader::get_value()
{
    char * s;
    char c;
    int n;

    switch (current_phrase) 
    {
        case 0 :    //   "Show convergence",
            c = get_char(value);
            if (c == 'y' || c == 'Y') 
            {
                monitor_conv = true;
            } else 
            {
                monitor_conv = false;
            }
            break;


        case 1 : //   "Weight matrix screen",
            scr_wgh = get_char(value);
            break;


        case 2 : //   "Mixing matrix screen",
            scr_mix = get_char(value);
            break;


        case 3 : //   "Unmixing matrix screen",
            scr_umx = get_char(value);
            break;

        case 4 : //   "Q matrix screen",
            scr_q = get_char(value);
            break;

        case 5 : //   "Lambda ^(-1/2) matrix screen",
            scr_lam = get_char(value);
            break;

        case 6 : //   "Average vector screen",
            scr_avg = get_char(value);
            break;

        case 7 : //   "Covariance matrix screen",
            scr_cov = get_char(value);
            break;

        case 8 : //   "Sphering matrix screen",
            scr_sph = get_char(value);
            break;

        case 9 : //   "File name",
            s  = get_str(value);
            n = strlen(s);

            if (n > 0) 
            {
                output_name = new char[n + 10];
                strcpy(output_name, s);
            }
            break;

        case 10 : //   "Weight matrix file",
            file_wgh = get_char(value);
            break;

        case 11 : //   "Mixing matrix file"
            file_mix = get_char(value);
            break;

        case 12 : //   "Unmixing matrix file",
            file_umx = get_char(value);
            break;

        case 13 : //   "Q matrix file",
            file_q = get_char(value);
            break;

        case 14 : //   "Lambda ^(-1/2) matrix file",
            file_lam = get_char(value);
            break;

        case 15 : //   "Average vector file",
            file_avg = get_char(value);
            break;

        case 16 : //   "Covariance matrix file",
            file_cov = get_char(value);
            break;

        case 17 : //  "Sphering matrix file"
            file_sph = get_char(value);
            break;

        case 18 :
            c = get_char(value);
            if (c == 'y' || c == 'Y') 
            {
                validate = true;
            } 
            else 
            {
                validate = false;
            }
            break;

        case 19 ://Independent component file name
            s = get_str(value);
            n = strlen(s);  
            if ( ( s != 0 ) && ( *s != 0 ) )
            {
               ICFileName = s;
            }
            else 
               ICFileName.clear();
            break;

        case 20:
            {
                m_binaryFileFormat = toupper( get_char( value ) );
            }
            break;

        default :
            break;

    }
}


#endif
// ICAOUTPUTREADER_CPP_UNIVERSITY_OF_OREGON_NIC
