#ifndef INFOMAX_H_UNIVERSITY_OF_OREGON_NIC
#define INFOMAX_H_UNIVERSITY_OF_OREGON_NIC

#define USE_DGEMM_MASTER
//#define USE_DGEMM_KGLASS

#include <iostream>

#include "LAPACKWrapper.h"
#include "MPIWrapper.h"

#include "Processor.h"
#include "InfomaxReader.h"
#include "nic_matrix.h"
#include "nic_vector.h"
#include "index_array.h"

#ifdef USE_TAU_PROFILING
#include "TAU.h"
#endif

extern "C" int dgemv_(char *, int *, int *, double *, double *, int *, double *, 
          int *, double *, double *, int *);

extern "C" int dgemm_(char *, char *, int *, int *, int *, double *,
          double *, int *, double *, int *, double *, double *, int *);

#ifdef  __OPENMP_ 
#include "omp.h"
#endif


class Infomax : public Processor 
{
public:
// --------------- data 
  nic_matrix* x;
  nic_matrix* x_training;
  int n_channels, n_observations;

  // -------------- convergence loop
  int steps, retries,  max_retries;
  int max_iterations;

  double tolerance;
  double delta, old_delta, init_delta;
  double delta_angle;

  bool                  exceeded;  // if weight limit is exceed restart

  // -------------- training loop
  int block_size;
  int n_blocks;
  double learning_rate;
  double annealing_scale;
  double annealing_degree;
  bool random_learning;

  double max_divergence;
  double divergence_factor;
  double max_weight;
  double weight_restart_factor;
  double min_learning_rate;

  nic_matrix* init_matrix;

  // -------------- procedure
  bool                  extended;

  nic_matrix* old_weights;
  nic_matrix* initial_weights;

  nic_matrix* weights_delta;
  nic_matrix* old_weights_delta;

  nic_matrix* u;
  nic_matrix* y;

  nic_vector  * bias;

  nic_matrix* bias_load;
  nic_matrix* temp_m;
  nic_vector* temp_v;

  IndexArray          * x_index;

private:
    nic_matrix* m_sph;

public:

    Infomax(
        nic_matrix* d, 
        InfomaxReader * reader, 
        nic_matrix* sph);
   ~Infomax();


    void allocate_space(InfomaxReader* reader);

    void load_init_matrix(InfomaxReader * reader, nic_matrix* sph);

    void init_nn();
    void run();

    void validate_load();
 protected:

    void search_for_weights_extended(); 

    void search_for_weights();

    double estimate_convergence(int pid);
    void anneal_learning_rate();

    bool training_loop(int pid);

    void set_u(int idx, int pid);

    void set_y(int pid);

    void update_W(int size, int pid);

    void update_bias(int pid);

    bool weight_exceeded(int pid, int it);

    void ABT(nic_matrix* A, nic_matrix* B, 
        nic_matrix* C);

    void xAB(nic_matrix* A, nic_matrix* B, 
        nic_matrix* C, double x);

};

#endif
// INFOMAX_H_UNIVERSITY_OF_OREGON_NIC
