#ifndef INFOMAXNEW_H_UNIVERSITY_OF_OREGON_NIC
#define INFOMAXNEW_H_UNIVERSITY_OF_OREGON_NIC

#include "InfomaxSettings.h"
#include "nic_matrix.h"
#include "MersenneTwister.h"
#include "LAPACKWrapper.h"
#include "index_array.h"

class InfomaxNew
{
public:
    InfomaxNew(
        const InfomaxSettings& settings,
        nic_matrix* data,
        unsigned long seed,
        bool verbose );
    ~InfomaxNew();

    void mixingMatrix( nic_matrix& matrix, bool spheringApplied = true );
    void setWhiteningMatrix( nic_matrix* matrix );
    
    void runInfomax();

    void posact( nic_matrix& data, nic_matrix& mixing );

private:
    // private methods
    void loadGuess();
    void searchForWeights();
    void initializeNeuralNet();
    bool trainingLoop();
    void setU( int index );
    void setY( );
    void updateW( int size );
    void updateBl( double* bl, double* y, int size );
    void updateTm( double* tm );
    void updateW( double* w, double* Dw, int size);
    void updateBias();
    bool weightExceeded();
    double estimateConvergence();
    void annealLearningRate();

    // private members
    InfomaxSettings m_settings;

    // data
    nic_matrix* m_data;
    nic_matrix m_mixingMatrix;
    nic_matrix* m_whiteningMatrix;
    nic_matrix* m_invWhiteningMatrix;
    int m_channels;
    int m_samples;
    nic_matrix* m_initialGuess;

    // algorithm structures for neural net
    double m_initDelta;
    double m_delta;
    double m_deltaAngle;
    double m_oldDelta;

    // a bunch of temorary data matrices
    nic_matrix m_biasLoad;
    nic_matrix m_tempM;
    nic_vector m_tempV;
    nic_matrix m_weights;
    nic_matrix m_oldWeights;
    nic_matrix m_xTraining;
    nic_matrix m_weightsDelta;
    nic_matrix m_oldWeightsDelta;
    nic_matrix m_u;
    nic_matrix m_y;
    nic_vector m_bias;
    IndexArray m_indexArray;
    int m_blocks;


    // function objects and functions
    MersenneTwister m_randNumGen;
    ContrastFunctions::ContrastFunction m_contrastFunction;

    bool m_verbose;
};

#endif
// INFOMAXNEW_H_UNIVERSITY_OF_OREGON_NIC
