#ifndef INFOMAXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define INFOMAXREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include "InfomaxReader.h"

// DEFINE FORM DATA NAMES HERE 
#define N_INFOMAX_PHRASES 17
char * INFOMAX_PHRASES[N_INFOMAX_PHRASES] = {
  "Convergence tolerance",
  "Maximum iterations",
  "Maximum retries",
  "Initialization Type",
  "User defined source",
  "User defined type",
  "User defined orientation",
  "Learn set size",
  "Learning rate",
  "Annealing degree",
  "Annealing scale",
  "Random Learning",
  "Maximum weight",
  "Maximum divergence",
  "Weight restart factor",
  "Divergence restart factor",
  "Minimum learning rate"
};


InfomaxReader::InfomaxReader( std::istream& inStream )
:   Reader( inStream )
{
    phrase = INFOMAX_PHRASES;
    n_phrases = N_INFOMAX_PHRASES;
}

void InfomaxReader::get_value() 
{
    char * s;
    char c;
    int n;

    switch (current_phrase) 
    {
        case 0 :
        tolerance = get_double(value);
        break;

    case 1 :
        max_iterations = get_int(value);
        break;

    case 2 :
        max_retries = get_int(value);
        break;

    case 3 :
        initialization_type = get_char(value);
        break;

    case 4 :
        s = get_str(value);
        n = strlen(s);
        if (n > 0) 
        {
            user_source = new char [n + 25];
            sprintf(user_source, "%s", s);
            if (!read_validate(user_source)) 
            {
                cout << "\nERROR in InfomaxReader, reading initialization matrix:"; 
                cout << "You do not have read permission for file:\n\n\t";
                cout << user_source;
                cout << "\n\nIf the file or the data path are\n";
                cout << "incorrect, please fix them in:\n\n";
                cout << "\tworkbench/forms/Infomax_process.form,\n\n";
                cout << "otherwise, please check with your system administrator.\n\n";
                exit(-18);
           }
        } 
        else 
        {
            user_source = NULL;
        }
        break;

    case 5 :
        user_type = get_char(value);
        break;

    case 6 :
        user_orient = get_char(value);
        break;

    case 7 :
        nn_block_size = get_int(value);
        break;

    case 8 :
        nn_learning_rate = get_double(value);
        break;

    case 9 :
        nn_annealing_degree = get_double(value);
        break;

    case 10 :
        nn_annealing_scale = get_double(value);
        break;

    case 11 :
        nn_random_learning = get_char(value);
        break;

    case 12 :
        nn_max_weight = get_double(value);
        break;

    case 13 :
        nn_max_divergence = get_double(value);
        break;

    case 14 :
        nn_weight_restart_factor = get_double(value);
        break;

    case 15 :
        nn_divergence_factor = get_double(value);
        break;

    case 16 :
        nn_min_learning_rate = get_double(value);
        break;

    default :
        break;
    }
}

void InfomaxReader::load_user_init(nic_matrix* init_mat) 
{
    if (user_type == 'B' || user_type == 'b') 
    {  // binary file
        if (user_orient == 'C' || user_orient == 'c') 
        {
            BinaryMatrix * mat_reader =
                new BinaryMatrix (init_mat, user_source);
        } else if (user_orient == 'O' || user_orient == 'o') 
        {
            BinaryMatrixT * mat_reader =
                new BinaryMatrixT (init_mat, user_source,false, 0);
        }
        } else if (user_type == 'A' || user_type == 'a') 
        {
            if (user_orient == 'C' || user_orient == 'c') 
            {
                ASCIIMatrix * mat_reader = 
                    new ASCIIMatrix (init_mat, user_source);
            } else if (user_orient == 'O' || user_orient == 'o') 
            {
                ASCIIMatrixT * mat_reader = 
                    new ASCIIMatrixT (init_mat, user_source);
            }
            } else 
        {
    // error 
    }
}


#endif
// INFOMAXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
