#ifndef MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include <fstream>
#include "DataReader.h"
#include "MatrixReader.h"
#include "nic_matrix.h"

using namespace std;

bool MatrixReader::loadMatrix( 
    nic_matrix& matrix, 
    Format format, 
    int rows,
    int columns,
    const string& fileName )
{
    // set the mode for file io
    ios::openmode mode = ios::in;
    if ( format != TEXT ) mode |= ios::binary;
    DataReader<double>::Endian endian;
    DataReader<double>::Format readerFormat;

    switch ( format )
    {
        case BIG:
            endian = DataReader<double>::BigEndian;
            readerFormat = DataReader<double>::Binary;
            break;
        case LITTLE:
            endian = DataReader<double>::LittleEndian;
            readerFormat = DataReader<double>::Binary;
            break;
        case NATIVE:
            endian = DataReader<double>::NativeEndian;
            readerFormat = DataReader<double>::Binary;
            break;
        case TEXT:
            endian = DataReader<double>::NativeEndian;
            readerFormat = DataReader<double>::Ascii;
            break;
        default:
            endian = DataReader<double>::NativeEndian;
            readerFormat = DataReader<double>::Ascii;
    }

    ifstream inputFile( fileName.c_str(), mode );
    if ( !inputFile.good() )
    {
        return false;
    }

    matrix.resize( rows, columns );

    DataReader<double> reader( inputFile, readerFormat, endian );
    
    double* d = matrix.data;
    int count = 0;
    int numValues = rows * columns;
    while ( (count < numValues) && reader.read( d ) )
    {
        ++d;
        ++count;
    }

    inputFile.close();
    
    return ( count == numValues );
}

#endif
// MATRIXREADER_CPP_UNIVERSITY_OF_OREGON_NIC
