#ifndef MATRIXWRITER_CPP_UNIVERSITY_OF_OREGON_NIC
#define MATRIXWRITER_CPP_UNIVERSITY_OF_OREGON_NIC

#include <fstream>
#include "DataWriter.h"
#include "MatrixWriter.h"

using namespace std;

bool MatrixWriter::writeMatrix(
    nic_matrix& matrix,
    MatrixWriter::Format format,
    const std::string& fileName )
{
    // set the mode for file io
    ios::openmode mode = ios::out;
    if ( format != TEXT ) mode |= ios::binary;

    DataWriter<double>::Endian endian;
    DataWriter<double>::Format writerFormat;

    switch ( format )
    {
        case BIG:
            endian = DataWriter<double>::BigEndian;
            writerFormat = DataWriter<double>::Binary;
            break;
        case LITTLE:
            endian = DataWriter<double>::LittleEndian;
            writerFormat = DataWriter<double>::Binary;
            break;
        case NATIVE:
            endian = DataWriter<double>::NativeEndian;
            writerFormat = DataWriter<double>::Binary;
            break;
        case TEXT:
            endian = DataWriter<double>::NativeEndian;
            writerFormat = DataWriter<double>::Ascii;
            break;
        default:
            endian = DataWriter<double>::NativeEndian;
            writerFormat = DataWriter<double>::Ascii;
    }

    ofstream outputFile( fileName.c_str(), mode );
    if ( !outputFile.good() )
    {
        return false;
    }

    DataWriter<double> writer( outputFile, writerFormat, endian );

    int numValues = matrix.rows * matrix.columns;
    int count = 0;
    double* d = matrix.data;
    while( (count < numValues) && writer.write( d ) )
    {
        ++d;
        ++count;
        if (( count % matrix.rows) == 0)
        {
            writer.linebreak();
        }
    }
    
    outputFile.close();
    return ( count == numValues );
}

#endif
// MATRIXWRITER_CPP_UNIVERSITY_OF_OREGON_NIC
