#ifndef NICSOBI_CPP_UNIVERSITY_OF_OREGON_NIC
#define NICSOBI_CPP_UNIVERSITY_OF_OREGON_NIC

#include "NicSobi.h"
#include "nic_matrix.h"
#include "Sobi.h"
#include "version.h"

int main( int argc, char** argv )
{
    // setup the parser
    SimpleCLParser parser( argc, argv );
    setupParser( parser );
    cout << parser.usageHeader() << endl;


    // determing the input data size and format
    int rows = parser.readIntOption( "-c" );
    int columns = parser.readIntOption( "-s" );

    MatrixReader::Format inputDataFormat = getInputFormat( parser );
    MatrixWriter::Format outputDataFormat = getOutputFormat( parser );

    // load the data
    nic_matrix inputData;
    MatrixReader::loadMatrix(
        inputData,
        inputDataFormat,
        rows, columns,
        parser.readStringOption( "-i" ) );

    // setup for Sobi computation
    int* tau = 0;
    int tauLength = 0;
    nic_matrix W;

    computeSobi( inputData, tau, tauLength, W );

    MatrixWriter::writeMatrix(
        W,
        outputDataFormat,
        parser.readStringOption( "-o" ) );

    delete[] tau;
    return 0;
}

void setupParser( SimpleCLParser& parser )
{
    bool isRequired = true;
    bool notRequired = false;

    // usage and copyright info
    string usage = "\nHiPerSAT Sobi v. 1.0 (r";
    usage += hipersatVersionString();
    usage +=  ")\n\n";
    usage += NIC_COPYRIGHT;
    parser.addUsageHeader( usage );

    // input file options
    parser.addOption( SimpleCLParser::STRING,
                      "-i", 
                      "input file name", 
                      isRequired );
    parser.addOption( SimpleCLParser::STRING,
                      "-if", 
                      "input data format ( big, little, native, text )",
                      isRequired );
    parser.addOption( SimpleCLParser::INTEGER,
                      "-c",
                      "number of channels",
                      isRequired );
    parser.addOption( SimpleCLParser::INTEGER,
                      "-s",
                      "number of samples",
                      isRequired );

    // output options
    parser.addOption( SimpleCLParser::STRING,
                      "-o",
                      "output file name for weight matrix",
                      isRequired );
    parser.addOption( SimpleCLParser::STRING,
                      "-of",
                      "output data format ( big, little, native, text )",
                      isRequired );

    // help and usage
    parser.addOption( SimpleCLParser::FLAG, "-h", "usage", notRequired );

    parser.parse();
    if ( parser.readFlagOption( "-h" ) )
    {
        cout << parser.usage() << endl;
        exit( 0 );
    }

    parser.checkRequired();
}

MatrixReader::Format getInputFormat( SimpleCLParser& parser )
{
    string formatString = parser.readStringOption( "-if" );
    MatrixReader::Format returnValue;
    if ( formatString == "big" )
    {
        returnValue = MatrixReader::BIG;
    }
    else if ( formatString == "little" )
    {
        returnValue = MatrixReader::LITTLE;
    }
    else if ( formatString == "native" )
    {
        returnValue = MatrixReader::NATIVE;
    }
    else if ( formatString == "text" )
    {
        returnValue = MatrixReader::TEXT;
    }
    else
    {
        cerr << "Invalid file type: " << formatString << endl;
        exit(1);
    }
    return returnValue;
}

MatrixWriter::Format getOutputFormat( SimpleCLParser& parser )
{
    MatrixWriter::Format format = MatrixWriter::TEXT;

    if ( parser.readFlagOption( "-of" ) )
    {
        string formatName = parser.readStringOption( "-of" );
        if ( formatName == "big" )
        {
            format = MatrixWriter::BIG;
        }
        else if ( formatName == "little" )
        {
            format = MatrixWriter::LITTLE;
        }
        else if ( formatName == "native" )
        {
            format = MatrixWriter::NATIVE;
        }
        else if ( formatName == "text" )
        {
            format = MatrixWriter::TEXT;
        }
        else 
        {
            cerr << "Invalid output type. Defaulting to text" << endl;
        }
    }
    return format;
}

#endif
// NICSOBI_CPP_UNIVERSITY_OF_OREGON_NIC
