#ifndef PROCESSREADER_CPP_UNIVERSITY_OF_OREGON_NIC
#define PROCESSREADER_CPP_UNIVERSITY_OF_OREGON_NIC

#include <iostream>
#include <cstring>

#include "PreprocessReader.h"

using namespace std;

bool write_validate(char * file_name);

// DEFINE FORM DATA NAMES HERE -------------------------------------------------
char * PREPROC_PHRASES[] = {
  "Preprocessing procedure",
  "Sphering matrix",
  "Sphering source",
  "Sphering type",
  "Sphering orientation",
  "Whitened data file",
};
const int N_PREPROC_PHRASES = sizeof( PREPROC_PHRASES ) / sizeof( PREPROC_PHRASES[ 0 ] );

// -----------------------------------------------------------------------------


PreprocessReader::PreprocessReader( istream& inStream ) :
   sphr_file_name( 0 ),
 	Reader( inStream )
{
    preprocess_flow = 0;
    phrase = PREPROC_PHRASES;
    n_phrases = N_PREPROC_PHRASES;
    flow_number = 0;

    load_sphere = false;    
    sph_matrix = NULL;
};


static void swapData( double* buf, size_t numElements, bool swap )
{        
//    fread( buf, sizeof( double ), numElements, fh );   // read in the master's data first
    
    if ( swap )
    {
        double* dptr = buf;
        
        while ( numElements-- > 0 )
        {
           unsigned char*   cptr = (unsigned char*) dptr;
           unsigned char    tmp;
           
           tmp = cptr[ 0 ];
           cptr[ 0 ] = cptr[ 7 ];
           cptr[ 7 ] = tmp;
           
           tmp = cptr[ 1 ];
           cptr[ 1 ] = cptr[ 6 ];
           cptr[ 6 ] = tmp;
           
           tmp = cptr[ 2 ];
           cptr[ 2 ] = cptr[ 5 ];
           cptr[ 5 ] =tmp;
           
           tmp = cptr[ 3 ];
           cptr[ 3 ] = cptr[ 4 ];
           cptr[ 4 ] = tmp;
           
           dptr++;
        }
    }
}

void PreprocessReader::load_sphering_matrix(int n)
{
    sph_matrix = new nic_matrix(n, n);
    sphr_file_type = std::toupper( sphr_file_type );
    
#if BIG_ENDIAN == BYTE_ORDER
    char            nativeType = 'B';
    bool                swap = ( sphr_file_type == 'L' );
#elif LITTLE_ENDIAN == BYTE_ORDER
    char            nativeType = 'L';
    bool                swap = ( sphr_file_type == 'B' );
#endif
    //cout<<"SWAP!!!" <<swap<<" sphere file type "<<sphr_file_type<<"\n";
    //cout << "SPHERING FILE NAME: " << sphr_file_name << endl;

    if (sphr_file_type == 'B' || sphr_file_type == 'L' || sphr_file_type == 'P') 
    {  // binary file
        if (sphr_orientation == 'C' || sphr_orientation == 'c') 
        {
            BinaryMatrix mat_reader( sph_matrix, sphr_file_name );
//            BinaryMatrix * mat_reader =
//            new BinaryMatrix (sph_matrix, sphr_file_name);
        } 
        else if (sphr_orientation == 'O' || sphr_orientation == 'o') 
        {
            BinaryMatrixT mat_reader(sph_matrix, sphr_file_name, false, 0);

//            BinaryMatrixT * mat_reader =
//            new BinaryMatrixT (sph_matrix, sphr_file_name, false, 0 );
        }

        swapData(sph_matrix->data, sph_matrix->rows*sph_matrix->columns,swap);
        //data swapped if required
        }
    else if (sphr_file_type == 'A') 
    {
        if (sphr_orientation == 'C' || sphr_orientation == 'c') 
        {
              ASCIIMatrix mat_reader(sph_matrix, sphr_file_name);
//            ASCIIMatrix * mat_reader = 
//                new ASCIIMatrix (sph_matrix, sphr_file_name);
        } 
        else if (sphr_orientation == 'O' || sphr_orientation == 'o') 
        {
              ASCIIMatrixT mat_reader(sph_matrix, sphr_file_name);
//            ASCIIMatrixT * mat_reader = 
//              new ASCIIMatrixT (sph_matrix, sphr_file_name);
        }

        swapData(sph_matrix->data, sph_matrix->rows*sph_matrix->columns,swap);
    }
    else 
    {
      // error 
    }
}


void PreprocessReader::get_value()
{
    char * s;
    char c;
    int n;

    switch (current_phrase) {
    case 0 :
      get_preprocess_flow(current_phrase);
      break;
    case 1 :
      c = get_char(value);
      if (c == 'f') {
	load_sphere = true;
      } else {
	load_sphere = false;
      }
      break;
    case 2 :
      s = get_str(value);
      n = strlen(s);

    if ( ( s != 0 ) && ( *s != 0 ) )
    {
        sphr_file_name = new char[ n + 25 ];
        sprintf( sphr_file_name, "%s", s );
        if ( !read_validate( sphr_file_name ) )
        {
            cout << "\nERROR: You do not have read permission for file:\n\n\t";
            cout << sphr_file_name;
            cout << "\n\nIf the file or the data path are\n";
            cout << "incorrect, please fix them in:\n\n";
            cout << "\tworkbench/forms/FastICA_process.form OR\n";
            cout << "\tworkbench/forms/Infomax_process.form\n\n";
            cout << "otherwise, please check with your system administrator.\n\n";
            exit(-19);
        }
    }
    else
    {
	    sphr_file_name = NULL;
    }
    break;
    
    case 3 :
      c = get_char(value);
      sphr_file_type = c;
      break;
    case 4 :
      c = get_char(value);
      sphr_orientation = c;
      break;
    
    case 5:
    {
        s = get_str(value);
        n = strlen(s);

        if ( ( s != 0 ) && ( *s != 0 ) )
        {
/*        
            write_validate fails if file does not already exist.
            
            if ( !write_validate( s ) )
            {
                cout << "\nERROR: You do not have write permission for file:\n\n\t";
                cout << s;
                cout << "\n\nIf the file or the data path are\n";
                cout << "incorrect, please fix them in your SignalCleaner .form file\n\n";
                exit( -19 );
            }
*/
            
            m_whitenedFilename = s;
        }
        else
        {
            m_whitenedFilename.clear();
        }
    }
    break;
    
    
    default :
      break;
    };
  }
  


void PreprocessReader::get_preprocess_flow(int current_phrase) {
    n_processes = get_int(value);

    if ( preprocess_flow ) delete [] preprocess_flow;
    preprocess_flow = new PREPROCESS_TYPE[n_processes];

    char * s = NULL;
 
    // If the user identifies preprocess procedures, n > 0
    if (n_processes != 0) {
 
      for (int i = 0; i < n_processes; i++)
      {
        readNextInstruction();
        s = get_str(instruction);
 
        if (!strcmp(s, "whiten")) {
          preprocess_flow[i] = WHITEN;
        } else if (!strcmp(s, "validate whitening")) {
          preprocess_flow[i] = VALIDATE_WHITENED;
        } else if (!strcmp(s, "store whitened data")) {
          preprocess_flow[i] = STORE_WHITENED;
        } else {
          // error !!!
        }
      }
    }
  };
#endif
// PROCESSREADER_CPP_UNIVERSITY_OF_OREGON_NIC
