#ifndef PREPROCESSOR_CPP_UNIVERSITY_OF_OREGON_NIC
#define PREPROCESSOR_CPP_UNIVERSITY_OF_OREGON_NIC

#include "Preprocessor.h"

Preprocessor::Preprocessor(
    nic_matrix * d, PreprocessReader * reader, int r,
    int c, bool useInfomaxSphering ) 
:   data(d), rank(r), n_cpus(c),
    m_useInfomaxSphering( useInfomaxSphering )
{
    nic_matrix::incrementReferenceCount( d );
    n_rows          = data->rows;
    n_columns       = data->columns;
    size            = data->size;

    n_processes     = reader->n_processes;
    preprocess_flow = reader->preprocess_flow; // I assume that I now own this

    avg             = NULL;
    cov             = NULL;
    q               = NULL;
    lambda          = NULL;

    m_whitenedFilename = reader->m_whitenedFilename;
    
    // the sphering matrix is init'ed to NULL in the PreprocessReader
    // if the reader does not load a matrix, it is returns a NULL
    if (reader->load_sphere) 
    {
        reader->load_sphering_matrix(n_rows);
    } 
    sph             = reader->sph_matrix;
}

Preprocessor::~Preprocessor() 
{
    nic_matrix::freeReference( data );
    delete [] preprocess_flow; // created by PreprocessReader
    if ( avg != NULL ) delete avg;
    if ( cov != NULL )delete cov;
    if ( q != NULL ) delete q;
    if ( lambda != NULL ) delete lambda;
    if ( sph != NULL ) delete sph;
}

void Preprocessor::run() 
{
    bool compute_sphere_req;
    PartitionedDataset storedata;

    for (int i = 0; i < n_processes; i++) 
    {
        if ( preprocess_flow ) // this may be incorrect cch
        {
            switch( preprocess_flow[i] ) 
            {
                case PreprocessReader::WHITEN :

                if (monitor_conv) 
                {
                    cout << "WHITENING DATA" << endl;
                }

                if (sph == NULL) 
                {
                    sph = new nic_matrix(n_rows, n_rows);
                    compute_sphere_req = true;
                } else 
                {
                    compute_sphere_req = false;
                }

                avg = new nic_vector(n_rows);
                cov = new nic_matrix(n_rows, n_rows);
                q = new nic_matrix(n_rows, n_rows);
                lambda = new nic_vector(n_rows);

                whiten_data(data, avg, cov, q, sph, lambda, compute_sphere_req, 
                    m_useInfomaxSphering );
                break;

            case PreprocessReader::VALIDATE_WHITENED :

                if (monitor_conv) 
                {
                    cout << "VALIDATING WHITENING PROCESS" << endl;
                }
                if (sph != NULL && rank == 0) 
                {
                    validate_whitening(data, avg, cov, q, sph, lambda, "." );
                } else 
                {
                    cout << "CANNOT VALIDATE WHITENED DATA, MISSING SPHERING MATRIX" << endl;
                }
                break;

            case  PreprocessReader::STORE_WHITENED :
                if (monitor_conv) 
                {
                    cout << "STORING WHITENED DATA" << endl;
                }

                storedata.storeFile(rank, data, m_whitenedFilename, n_cpus);
                break;

            default :
                break;
            }
        }
    }
}

void Preprocessor::validate_load() 
{
    cout << "PREPROCESSOR LOAD VALIDATION --------------------------------" << endl;
    cout << "\tnumber of processes        = " << n_processes << endl;
    cout << "\tpreprocess flow:" << n_processes << endl;
    for (int i = 0; i < n_processes; i++) 
    {
        cout << "\t\t" << preprocess_flow[i] << endl;;
    }

    cout << "\tdata size                  =  (" << n_rows << ", " << n_columns;
    cout << ", " <<  size << ")" << endl;

    
    cout << "DATA SETS: first five elements from first column" << endl;

    cout << "\tdata @ " << data << ": ";
    for (int i = 0; i < 5; i++) 
    {
        cout << data->data[i] << " ";
    }
    cout << endl;

    cout << "\tavg @ " << avg;
    if (avg != NULL) 
    {
        cout << ": ";
        for (int i = 0; i < 5; i++) 
        {
	        cout << avg->data[i] << " ";
        }
    }
    cout << endl;

    cout << "\tcov @ " << cov;
    if (cov != NULL) 
    {
        cout << ": ";
        for (int i = 0; i < 5; i++) 
        {
	        cout << cov->data[i] << " ";
        }
    }
    cout << endl;

    cout << "\tsph @ " << sph;
    if (sph != NULL) 
    {
        cout << ": ";
        for (int i = 0; i < 5; i++) 
        {
	        cout << sph->data[i] << " ";
        }
    }
    cout << endl;

    cout << "\tq @ " << q;
    if (q != NULL) 
    {
        cout << ": ";
        for (int i = 0; i < 5; i++) 
        {
	        cout << q->data[i] << " ";
        }
    }
    cout << endl;

    cout << "\tlambda @ " << lambda;
    if (lambda != NULL) 
    {
        cout << ": ";
        for (int i = 0; i < 5; i++) 
        {
	        cout << lambda->data[i] << " ";
        }
    }
    cout << endl;

    cout << "END PREPROCESSOR LOAD VALIDATION ----------------------------" << endl;
}

#endif
// PREPROCESSOR_CPP_UNIVERSITY_OF_OREGON_NIC
