#ifndef SORTCOMPONENTS_CPP_UNIVERSITY_OF_OREGON_NIC
#define SORTCOMPONENTS_CPP_UNIVERSITY_OF_OREGON_NIC

#include "SortComponents.h"
#include "MatrixOperations.h"

#include <algorithm>
#include <vector>

using namespace std;

bool comparePair( pair< int, double > l, pair< int, double > r )
{
    return l.second > r.second;
}

void sortComponents( nic_matrix& data, nic_matrix& w, nic_matrix& s )
{
    vector< pair< int, double > > meanVariances;

    nic_matrix tempS = s;
    multiply( w, tempS );

    nic_matrix wInv( w.rows, w.columns );
    invert( tempS, wInv );


    // compute the mean variances of the projected data
    for ( int k = 0; k < data.rows; ++k )
    {
        double mean = 0;
        for ( int i = 0; i < data.columns; ++i )
        {
            double variance = 0.0;
            for ( int j = 0; j < data.rows; ++j )
            {
                variance += wInv( j, k ) * wInv(j, k ) * data( j, i ) * data(j, i );
            }
            mean += variance;
        }
        pair< int, double > result( k, mean );
        meanVariances.push_back( result );
    }

    sort( meanVariances.begin(), meanVariances.end(), comparePair );

    vector< int > ordering( data.rows );
    for ( int i = 0; i < data.rows; ++i )
    {
        ordering[i] = i;
    }

    double temp;
    int tempInt;

    for ( int i = 0; i < data.rows; ++i )
    {
        // find the location of the vector to move
        int toMove = meanVariances[i].first;
        int location = toMove;
        while ( toMove != ordering[ location ] )
        {
            location = ordering[ location ];
        }

        if ( i != location )
        {
            // now we swap i row with location row
            for ( int j = 0; j < data.columns; ++j )
            {
                temp = data( location, j );
                data( location, j ) = data( i, j );
                data( i, j ) = temp;
            }
            for ( int j = 0; j < data.rows; ++j )
            {
                temp = w( location, j );
                w( location, j ) = w( i, j );
                w( i, j ) = temp;
            }
            tempInt = ordering[ location ];
            ordering[ location ] = ordering[ i ];
            ordering[ i ] = tempInt;
        }
    }
}


#endif
// SORTCOMPONENTS_CPP_UNIVERSITY_OF_OREGON_NIC
