#ifndef COVARIANCE_CPP_UNIVERSITY_OF_OREGON_NIC
#define COVARIANCE_CPP_UNIVERSITY_OF_OREGON_NIC

#include "covariance.h"

#include <iostream>
using namespace std;

#undef __DISTRIBUTED

void
covariance(nic_matrix* data_, 
	   nic_matrix* cov_,
	   nic_vector* avg_)
{
    double* data = data_->data;
    double* avg = avg_->data;
    double* cov = cov_->data;

    double* d_ptr = NULL;
    double* a_ptr = NULL;
    double* c_ptr = NULL;

    int ch = data_->rows;
    int obs = data_->columns;
    int size = cov_->size;


    int total_obs = obs;
    #ifdef __DISTRIBUTED
    total_obs = data_->total_columns;
    #endif

    double recip_n = 1./((double)total_obs);
    double recip_n_1 = 1./((double)total_obs - 1.);

    #ifdef __DISTRIBUTED
    nic_matrix cov_temp(cov_->rows, cov_->columns);
    nic_vector avg_temp(avg_->n);
    #endif

    for (int i = 0; i < obs; i++) 
    {
        d_ptr = data + i * ch;

        for (int j = 0; j < ch; j++) 
        {
            avg[j] += d_ptr[j];
            for (int k = j; k < ch; k++) 
            {
	            *(cov + k + j * ch) += d_ptr[j] * d_ptr[k];
            }
        }
    } // finish data load

    #ifdef DEBUGCODE
    double cov_check[ch*ch];

    for(int j=0;j<ch;j++) 
    {
        for(int k=0; k<ch; k++) 
        {
            cov_check[k+j*ch] = *(cov + k + j*ch); 
        }
    }
    #endif

    #ifdef __DISTRIBUTED
    // In the distributed implementation, each processor has a partial
    // set of the sums and sums squared, they need to be accumulated
    // across all processors

    MPI_Allreduce(avg_->data, avg_temp.data, 
        ch, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
    MPI_Allreduce(cov_->data, cov_temp.data, 
        size, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

    cov_->swap_matrices(&cov_temp);
    avg_->swap_vectors(&avg_temp);

    cov = cov_->data;
    avg = avg_->data;

    // at this point, all processors have the total sum and sums of sqs
    #endif

    #ifdef DEBUGCODE
    cout<<"Are They Equal ? ";
    for(int j=0;j<ch;j++) 
    {
        for(int k=0; k<ch; k++) 
        {
		    cout<<(cov_check[k+j*ch]==cov[k+j*ch])<<" ";
	    }		
    }
    cout<<"\n";

    cout<<"Arbit Var : "<<recip_n_1<<" "<<recip_n<<"\n";
    #endif

    for (int j = 0; j < ch; j++) 
    {
        for (int k = j; k < ch; k++) 
        {
            *(cov + k + j * ch) -= avg[j] * avg[k] * recip_n;
            *(cov + j + k * ch) = *(cov + k * ch + j );
        }
    }

    for (int j = 0; j < ch; j++) 
    {
        avg[j] *= recip_n;
        for (int k = j; k < ch; k++) 
        {
            *(cov + k + j * ch) *= recip_n;
            *(cov + j + k * ch) = *(cov + k + j * ch);
        }
    }

    for (int i = 0; i < ch; i++) 
    {
        avg[i] *= recip_n;
    }

}

#endif
// COVARIANCE_CPP_UNIVERSITY_OF_OREGON_NIC
