#ifndef CUBIC_CPP_UNIVERSITY_OF_OREGON_NIC
#define CUBIC_CPP_UNIVERSITY_OF_OREGON_NIC

#include "cubic.h"

using namespace std;

// Purpose: compute the covariance matrix and its eigenvalues
void
cubic(nic_matrix* x, nic_vector* weight, 
		  nic_vector* weight_plus) 
{
    int rank = x->rank;

    int n_channels = x->rows;
    int n_samples = x->columns;
    int total_samples = x->total_columns;

    double * whitened_x = x->data;
    double * w = weight->data;
    double * w_plus = weight_plus->data;

    double * x_i = NULL;
    double wTx = 0;

    for (int i = 0; i < n_samples; i++) 
    {
        wTx = 0;
        x_i = whitened_x + i * n_channels; // x is the ith column

        // compute wTx with x = ith column
        for (int j = 0; j < n_channels; j++) 
        {
            wTx += w[j] * x_i[j];
        }

        for (int j = 0; j < n_channels; j++) 
        {   // E(x g(wTx))
            w_plus[j] += (wTx*wTx*wTx * x_i[j]);
        }
    }

    for (int j = 0; j < n_channels; j++) 
    {   // E(x g(wTx))
        w_plus[j] /= total_samples;
    }

    if (rank == 0) 
    {
        for (int j = 0; j < n_channels; j++) 
        {   // E(x g(wTx))
            w_plus[j] -=  (3 * w[j]);
        }
    }
}
#endif
// CUBIC_CPP_UNIVERSITY_OF_OREGON_NIC
