#ifndef NIC_MATRIX_CPP_UNIVERSITY_OF_OREGON_NIC
#define NIC_MATRIX_CPP_UNIVERSITY_OF_OREGON_NIC

#include "nic_matrix.h"
#include "nic_vector.h"

#include <iostream>

nic_matrix::nic_matrix(int r, int c, int t_r, int t_c, 
    int rk, int n_c) 
:   rows(r), columns(c), size(r*c), 
    rank(rk), n_cpus(n_c), total_size(t_c*t_r), total_columns(t_c), 
    total_rows(t_r) 
{
    referenceCount = 0;
    if (total_rows == 0) total_rows = rows;
    if (total_columns == 0) total_columns = columns;
    if (total_size == 0) total_size = size;

    if (size == 0) 
    {
        data = NULL;
    } 
    else 
    {
        data = new double[size];
        double* d = data;
        for ( int i = 0; i < size; ++i ) 
        {
            *d++ = 0;
        }
    }
}

void nic_matrix::resize( int r, int c, int t_r, int t_c, int rk, int n_c )
{
    rows = r;
    columns = c;
    size = r*c;
    rank = rk;
    n_cpus = n_c;

    if ( 0 == t_r ) total_rows = r;
    else total_rows = t_r;

    if ( 0 == t_c ) total_columns = columns;
    else total_columns = t_c;

    if ( data != NULL ) delete [] data;
    if ( size != 0 )
    {
        data = new double[size];
        double* d = data;
        for ( int i = 0; i < size; ++i )
        {
            *d++ = 0;
        }
    }
}


nic_matrix::nic_matrix(nic_matrix & m) 
:   rows(m.rows), columns(m.columns), size(m.size),
    rank(m.rank), n_cpus(m.n_cpus), total_columns(m.total_columns),
    total_rows(m.total_rows), total_size(m.total_size)
{
    referenceCount = 0;
    data = new double[size];
    memcpy(data, m.data, sizeof(double) * size);
}

nic_matrix::~nic_matrix()
{
    // this may not be safe
    if ( data != NULL ) delete[] data;
}

void
nic_matrix::incrementReferenceCount( nic_matrix* matrix )
{
      ++(matrix->referenceCount);
}

void
nic_matrix::freeReference( nic_matrix* matrix )
{
      --(matrix->referenceCount);
      if ( matrix->referenceCount == 0 )
      {
          delete matrix;
      }
}

void
nic_matrix::zero_matrix() 
{
  double* ptr = data;

  for (int i = 0; i < size; i++) {
    *ptr++ = 0;
  }
}

void
nic_matrix::identity_matrix() 
{
  double* ptr = data;

  if (rows == columns) {
    for (int i = 0; i < size; i++) {
      *ptr++ = 0;
    }
    for (int i = 0; i < rows; i++) {
      *(data + i * rows + i) = 1;
    }
    
  } else {
    cout << "ERROR: (nic_matrix::identity_matrix)-- Matrix must be square" << endl;
    exit(-8);
  }

}

void
nic_matrix::random_matrix(double min, double max) 
{
  double* d = data;
  double delta = max - min;

  for (int i = 0; i < size;  i++) {
    *d++ = (double)(min + delta * rand()/((double)RAND_MAX));
  }
};

void
nic_matrix::constant_matrix(double& v) 
{
  double* ptr = data;

  for (int i = 0; i < rows; i++) {
    for (int j = 0; j < columns; j++) {
      *ptr++ = v;
    }
  }
};

void 
nic_matrix::add_column(nic_vector* v_, int i)
{
  double* d = data + i * rows;
  double* v = v_->data;

  for (int i = 0; i < rows; i++)
  {
    *d = *v;
    ++d;
    ++v;
    // *d++ = *v++; cch what's above is much clearer!
  }
}


void
nic_matrix::add_row(nic_vector* v_, int i)
{
  double* d = data + i;
  double* v = v_->data;

  for (int i = 0; i < rows; i++)
  {
    *d = *v++;
    d += rows;
  }
}




void 
nic_matrix::add_diagonal(nic_vector* v_) 
{
  double* d = data;
  double* v = v_->data;

  for (int i = 0; i < rows; i++) {
    *d = *v++;
    d += rows + 1;
  }
};

void nic_matrix::swap_matrices(nic_matrix * A) 
{
    double* d = data;
    data = A->data;
    A->data = d;
}

// i is row
// j is column
double& nic_matrix::operator()(int i, int j)
{
    assert( ( i >= 0 ) && ( i < rows ) );
    assert( ( j >= 0 ) && ( j < columns ) );
    
    return data[ i + (j * rows)];
}

void nic_matrix::operator=(nic_matrix & B) 
{
    if (data != NULL) 
    {
        delete[] data;
    }
    rows = B.rows;
    columns = B.columns;
    size = B.size;
    total_size = B.total_size;
    total_rows = B.total_rows;
    total_columns = B.total_columns;
    n_cpus = B.n_cpus;
    rank = B.rank;

    data = new double[size];

    double* d = data;
    double* b = B.data;

    for (int i = 0; i < size; i++) 
    {
        *d++ = *b++;
    }
}

#endif
// NIC_MATRIX_CPP_UNIVERSITY_OF_OREGON_NIC
