#ifndef NIC_VECTOR_CPP_UNIVERSITY_OF_OREGON_NIC
#define NIC_VECTOR_CPP_UNIVERSITY_OF_OREGON_NIC

#include "nic_vector.h"
#include "nic_matrix.h"
#include <cmath>

using namespace std;


/***************************************************************
Test the template parameter for valid types
 **************************************************************/
nic_vector::nic_vector() : n (0), data(NULL) {}

nic_vector::nic_vector(int n_) : n (n_), data(new double[n]) 
{
    double* d = data;

    for (int i = 0; i < n; i++) 
    {
        *d++ = 0; 
    }
}

nic_vector::nic_vector(nic_vector &  v) 
:   n(v.n), data(new double[v.n]) 
{
    memcpy(data, v.data, sizeof(double) * n);
}

nic_vector::~nic_vector()
{
    if ( data ) delete[] data;
}

void 
nic_vector::swap_vectors(nic_vector * v) 
{
    double* temp = data;
    data = v->data;
    v->data = temp;
}

void 
nic_vector::zero_vector() 
{
    double* temp = data;
    for (int i = 0; i < n; i++) 
    {
        *temp = 0;
        ++temp;
    }
}


void 
nic_vector::constant_vector(double& v) 
{
    double* temp = data;
    for (int i = 0; i < n; i++) 
    {
        *temp++ = v;
    }
}

// UPDATE THIS
// Casting everything to type T sucks. Should probably use RTTI to
// determine if complex and, if so, size *=2, cast to type of 
// complex and preceed
void
nic_vector::normalize() 
{
    double norm = 0;
    int i = 0;

    double * d = (double *) data;

    for (i = 0; i < n; i++) 
    {
        norm += d[i] * d[i];
    }
    norm = sqrt(norm);

    for (i = 0; i < n; i++) 
    {
        d[i] /= norm;
    }
}

/****

The A is assumed to be normalized, 

v[i] = v[i] - A*v

*/

void
nic_vector::orthogonalize(nic_matrix* A) 
{ 
    int i = 0, j = 0;
    double * temp = (double *)new double[n];
    double * a = (double *)(A->data);
    double * d = (double *)data;

    for (i = 0; i < n; i++) 
    {
        temp[i] = d[i];
    }

    for (i = 0; i < n; i++) 
    {
        for (j = 0; j < n; j++) 
        {
            temp[i] -= a[i + n * j] * d[j];
        }
    }

    for (i = 0; i < n; i++) 
    {
        d[i] = temp[i];
    }

    delete [] temp;
}

// THIS SHOULD ACCEPT A R.N. GENERATING FUNCTION FOR THE PARTICULAR DATA TYPE

void 
nic_vector::random_vector(double min, double max) 
{
    double* d = data;
    double delta = max - min;

    for (int i = 0; i < n;  i++) 
    {
        *d++ = (double)(min + delta * rand()/((double)RAND_MAX));
    }
}

void nic_vector::operator=(nic_vector& v) 
{
    if ( this != &v )
    {
      delete [] data;
      data = new double[ v.n ];

      memcpy(data, v.data, sizeof(double)*n);
    }
}

bool nic_vector::operator==(nic_vector& v) 
{
    bool rtn = true;
    double* d = v.data;

    if (v.n != n) 
    {
        rtn = false;
    } 
    else 
    {
        for (int i = 0; i < n && rtn != false; i++) 
        {
            if (d[i] != data[i]) 
            {
                rtn = false;
            }
        }
    }
    return rtn;
}

#endif
// NIC_VECTOR_CPP_UNIVERSITY_OF_OREGON_NIC
