#ifndef READER_CPP_UNIVERSITY_OF_OREGON_NIC
#define READER_CPP_UNIVERSITY_OF_OREGON_NIC

#include "reader.h"

Reader::Reader( std::istream& inStream ) 
:   m_inStream( inStream ) 
{}

bool Reader::readNextInstruction()
{
    return m_inStream.getline( instruction, 256 );
}

template <class R>
char* Parser<R>::trim( char* buf )
{
    char*	firstNB = 0;
    char*	lastNB = 0;
    char*	p = buf;
    int		len = strlen( buf );
    
    while ( len-- > 0 )
    {
        if ( !isspace( *p ) )
        {
            if ( firstNB == 0 )
            {
                firstNB = p;
            }
            
            lastNB = p;
        }
        
        ++p;
    }
    
    if ( lastNB == 0 )
    {
        *buf = 0;
    }
    else
    {
        *( lastNB + 1 ) = 0;
        char* tempBuffer = new char[ lastNB - firstNB + 2 ];
        strcpy( tempBuffer, firstNB );
        strcpy( buf, tempBuffer );
        delete [] tempBuffer;
    }
    
    return buf;
}

template <class R>
bool Parser<R>::is_instruction()
{
    int 	i = 0;
    bool 	done = false;


    trim( R::instruction );
    int		ilen = strlen( R::instruction );

    if ( ( ilen == 0 ) ||
         ( ( ilen >= 2 ) && 
            ( R::instruction[ 0 ] == '/' && R::instruction[ 1 ] == '/' ) ) )
    {
    }
    else
    {
        char*	colonLoc = strchr( R::instruction, ':' );
    
        if ( colonLoc == 0 )
        {
            //	cerr << "#	>>> Missing colon (:) for line: /" 
            // << R::instruction << "/" << endl;
        }
        else
        {
            //cerr << "#>>> Line: /" << R::instruction << "/" << endl;

            *colonLoc = 0;
            char*		afterColon = colonLoc + 1;

            trim( R::instruction );
            //cerr << "#	>>> Inst: /" << R::instruction << "/" << endl;
        
            while ( i < R::n_phrases )
            {
                if ( strcmp( R::instruction, R::phrase[ i ] ) == 0 )
                {
                    R::value = afterColon;
                    trim( R::value );
            
                    //cerr << "#	>>> Value: /" << R::value << "/" << endl;
            
                    done = true;
                    break;
                }
                ++i;
            }
        }
    }

    R::current_phrase = i;

    //cerr << "#>>> Phrase: " << R::current_phrase << endl;

    return done; 
}

template <class R>
Parser<R>::Parser( std::istream& inStream )
:   R( inStream )
{
    inStream.clear();
    inStream.seekg( 0, std::ios_base::beg );
    inStream.clear();
     
    parse( inStream );
}

template <class R>
void Parser<R>::parse( std::istream& inStream )
{
    while( inStream )
    {
        if ( R::readNextInstruction() )
        {
            if ( is_instruction() )
            {
                R::get_value();
            }
        }
    }
}

template <class R>
char Parser<R>::get_char(char * value) 
{
    int i = 0;

    while(isspace(value[i]))
    {
        ++i;
    }

    return value[i];
}

template <class R>
char* Parser<R>::get_str(char * value)
{
    // if the string is not delimited by "", get contiguous characters
    // otherwise, get close quote
    int i = 0;
    int len = 0;
    char * str;
    char * end;

    while(isspace(value[i])) 
    {
        ++i;
    }

    if (value[i] == '\"') 
    {
        // string is delimited by ""
        ++i;
        str = &value[i];
        while(value[i] != '\"') 
        {
            ++i;
        }
        value[i] = '\0';
    } else 
    {
        // string is not delimited
        str = &value[i];
        if (str[0] == '\"') 
        {
            len = 0;
            str[0] = '\0';
        } else 
        {
            strtok(str, " ");
            len = strlen(value);

            if (str[len - 2] == 10) 
            {
                str[len - 2] = '\0';
            }
        }
    }

    return str;
}

template <class R>
int Parser<R>::get_int(char * value) 
{
    int i = 0;
    int num;

    strtok(value, " ");
    num = atoi(value);

    return num;    
}

template <class R>
double Parser<R>::get_double(char * value) 
{
    int i = 0;
    double num;

    strtok(value, " ");
    num = atof(value);

    return num;
}

template <class R>
char** Parser<R>::get_string_set(char * value, int n) 
{
    char**  rtn = new char*[n];
    char * s = NULL;

    for (int i = 0; i < n; i++) 
    {
        R::readNextInstruction();
        s = get_str(R::instruction);
        rtn[i] = new char[strlen(s) + 1];
        strcpy(rtn[i], s);
    }

    return rtn;
}

#ifdef INSTANTIATE_TEMPLATES
#include "CleanerReader.h"
#include "PreprocessReader.h"
#include "FastICAReader.h"
#include "ICAOutputReader.h"
#include "InfomaxReader.h"

template class Parser<CleanerReader>;
template class Parser<PreprocessReader>;
template class Parser<FastICAReader>;
template class Parser<ICAOutputReader>;
template class Parser<InfomaxReader>;
#endif

#endif
// READER_CPP_UNIVERSITY_OF_OREGON_NIC
