/*******************************************************************************
template <class Struct> struct stack                                            
                                                                                
AUTHOR: Kevin Glass                                                             
        NeuroInformatics Center,                                                
        University of Oregon                                                    
        kglass@nic.uoregon.edu                                                  
                                                                                
Purpose:                                                                        
Store structures of type Struct in a LIFO data structure.                       
                                                                                
Note:                                                                           
                                                                                
Validated:                                                                      
2/25/04 -- stack_test.cc                                                        
********************************************************************************/

#ifndef __STACK_H
#define __STACK_H

#include <cstdlib>

template <class Struct>
struct stack 
{
    Struct * head;
    int      size;

    stack() : head(NULL), size(0){};
    ~stack()
    {
        Struct * temp = head;
        while (temp != NULL) 
        {
          head = head->next;
          delete head;
          temp = head;
        }
    }

    void push(Struct * s) 
    {
        if (head == NULL) 
        {
            head = s;
        } else 
        {
            s->next = head;
            head->last = s;
            head = s;
        }
        size++;
    }

    Struct * pop()
    {
        Struct * s = head;

        if (head == NULL) 
        {
            s = NULL;
        } else 
        {
            head = head->next;
            if (head != NULL) 
            {
	            head->last = NULL;
            }
            s->next = NULL;
            s->last = NULL;
            --size;
        }

        return s;
    }

    void validate()
    {
        Struct * temp = head;
        if (head == NULL) 
        {
            cout << "STACK IS EMPTY" << endl;
            return;
        }

        cout << "FORWARD:" <<endl;
        while(temp->next != NULL) 
        {
            cout << "(" << temp->key << ", " << temp->value << ") ";
            temp = temp->next;
        }

        cout  << "(" << temp->key << ", " << temp->value << ") " << endl;

        cout << "REVERSE:" <<endl;
        while(temp != NULL) 
        {
            cout << "(" << temp->key << ", " << temp->value << ") ";
            temp = temp->last;
        }
        cout << "\n\n\n" << endl;
    }
};

#endif
